
#include "basewindow.h"
#include <gtksourceview/gtksourcebuffer.h>
#include <gtksourceview/gtksourceview.h>
#include "document.h"
#include "metadata.h"
#include "uimanager.h"
#include "config.h"
#include "settings.h"

struct _BaseWindowPrivate {
	GtkAboutDialog* about_dialog;
	GtkHPaned* pane;
	GtkHBox* searchbar;
	SearchEntry* searchentry;
	GtkStatusbar* statusbar;
	GtkStatusbar* secondary_statusbar;
	unsigned int flash_timeout;
	double hscroll;
	double vscroll;
};
#define BASE_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_BASE_WINDOW, BaseWindowPrivate))
enum  {
	BASE_WINDOW_DUMMY_PROPERTY,
};
static GtkClipboard* base_window_clipboard;
static const char* BASE_WINDOW_version = "0.3.0";
static const char* BASE_WINDOW_authors[] = {"Daniel Borgmann <daniel.borgmann@gmail.com>", NULL};
static gboolean base_window_real_is_document (BaseWindow* self, char* uri);
static gboolean base_window_scroll_timeout (BaseWindow* self);
static gboolean __lambda0 (GtkWidget* o, GdkEventFocus* e, BaseWindow* self);
static void base_window_flash_statusbar (BaseWindow* self, char* str);
static void base_window_flash_statusbar_clear (BaseWindow* self);
static gboolean base_window_flash_statusbar_timeout (BaseWindow* self);
static void base_window_show_about_dialog (BaseWindow* self);
static gboolean base_window_close (BaseWindow* self);
static void base_window_on_undo_redo_changed (GObject* o, gboolean test, BaseWindow* self);
static void base_window_on_document_mark_set (GObject* o, GtkTextIter* location, GtkTextMark* mark, BaseWindow* self);
static void base_window_on_menu_item_select (GtkMenuItem* proxy, BaseWindow* self);
static void base_window_on_menu_item_deselect (GtkMenuItem* proxy, BaseWindow* self);
static void base_window_on_connect_proxy (GtkUIManager* manager, GtkAction* action, GtkWidget* proxy, BaseWindow* self);
static void base_window_on_disconnect_proxy (GtkUIManager* manager, GtkAction* action, GtkWidget* proxy, BaseWindow* self);
static void base_window_on_searchentry_search_activated (GObject* o, SearchEventArgs* args, BaseWindow* self);
static gboolean base_window_on_searchentry_key_press (GObject* o, GdkEventKey* key, BaseWindow* self);
static gboolean base_window_on_searchentry_focus_out (GObject* o, GdkEventAny* e, BaseWindow* self);
static void base_window_on_searchentry_closed (GObject* o, BaseWindow* self);
static void base_window_on_action_activated (GObject* o, GtkAction* action, BaseWindow* self);
static gboolean base_window_on_close (GObject* o, GdkEventConfigure* e, BaseWindow* self);
static GObject * base_window_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void base_window_dispose (GObject * obj);

GtkClipboard* base_window_get_clipboard ()
{
	return base_window_clipboard;
}


static gboolean base_window_real_is_document (BaseWindow* self, char* uri)
{
	g_return_val_if_fail (IS_BASE_WINDOW (self), FALSE);
	return TRUE;
}


gboolean base_window_is_document (BaseWindow* self, char* uri)
{
	BASE_WINDOW_GET_CLASS (self)->is_document (self, uri);
}


gboolean base_window_save_scroll_position (BaseWindow* self)
{
	g_return_val_if_fail (IS_BASE_WINDOW (self), FALSE);
	if (self->document_view == NULL) {
		return FALSE;
	}
	long hscroll = gtk_adjustment_get_value (gtk_scrolled_window_get_hadjustment (self->scrolled_window));
	long vscroll = gtk_adjustment_get_value (gtk_scrolled_window_get_vadjustment (self->scrolled_window));
	char* __temp3 = NULL;
	metadata_manager_set (document_get_metadata (document_view_get_document (self->document_view)), "hscroll", (__temp3 = g_strdup_printf ("%li", hscroll)));
	(__temp3 == NULL ? NULL : (__temp3 = (g_free (__temp3), NULL)));
	char* __temp4 = NULL;
	metadata_manager_set (document_get_metadata (document_view_get_document (self->document_view)), "vscroll", (__temp4 = g_strdup_printf ("%li", vscroll)));
	(__temp4 == NULL ? NULL : (__temp4 = (g_free (__temp4), NULL)));
	return FALSE;
}


void base_window_restore_scroll_position (BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	self->priv->hscroll = metadata_manager_get_long (document_get_metadata (document_view_get_document (self->document_view)), "hscroll");
	self->priv->vscroll = metadata_manager_get_long (document_get_metadata (document_view_get_document (self->document_view)), "vscroll");
	g_timeout_add (50, base_window_scroll_timeout, (gpointer) self);
}


static gboolean base_window_scroll_timeout (BaseWindow* self)
{
	g_return_val_if_fail (IS_BASE_WINDOW (self), FALSE);
	double __temp7;
	double __temp6;
	if (self->priv->vscroll <= (g_object_get (G_OBJECT (gtk_scrolled_window_get_vadjustment (self->scrolled_window)), "upper", &__temp6, NULL), __temp6) && self->priv->hscroll <= (g_object_get (G_OBJECT (gtk_scrolled_window_get_hadjustment (self->scrolled_window)), "upper", &__temp7, NULL), __temp7)) {
		gtk_adjustment_set_value (gtk_scrolled_window_get_vadjustment (self->scrolled_window), self->priv->vscroll);
		gtk_adjustment_set_value (gtk_scrolled_window_get_hadjustment (self->scrolled_window), self->priv->hscroll);
		return FALSE;
	} else {
		return TRUE;
	}
}


void base_window_setup_ui_manager (BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	GtkUIManager* __temp10 = NULL;
	self->ui_manager = (__temp10 = GTK_UI_MANAGER (g_object_new (TYPE_CUSTOM_UI_MANAGER, NULL)), (self->ui_manager == NULL ? NULL : (self->ui_manager = (g_object_unref (self->ui_manager), NULL))), __temp10);
	g_signal_connect_object (self->ui_manager, "post-activate", (GCallback) base_window_on_action_activated, self, 0);
	g_signal_connect_object (self->ui_manager, "connect-proxy", (GCallback) base_window_on_connect_proxy, self, 0);
	g_signal_connect_object (self->ui_manager, "disconnect-proxy", (GCallback) base_window_on_disconnect_proxy, self, 0);
	char* __temp11 = NULL;
	gtk_ui_manager_add_ui_from_file (self->ui_manager, (__temp11 = g_strconcat (config_get_datadir (), "scratchpad-ui.xml", NULL)), NULL);
	(__temp11 == NULL ? NULL : (__temp11 = (g_free (__temp11), NULL)));
	gtk_window_add_accel_group (GTK_WINDOW (self), gtk_ui_manager_get_accel_group (self->ui_manager));
}


void base_window_update_menu_state (BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	GtkToggleAction* ta = NULL;
	ta = GTK_TOGGLE_ACTION (gtk_ui_manager_get_action (self->ui_manager, "/menubar/document/word-wrap"));
	if (g_utf8_collate (metadata_manager_get (document_get_metadata (document_view_get_document (self->document_view)), "word-wrap"), "true") == 0) {
		gtk_toggle_action_set_active (ta, TRUE);
	} else {
		if (g_utf8_collate (metadata_manager_get (document_get_metadata (document_view_get_document (self->document_view)), "word-wrap"), "true") == 0) {
			gtk_toggle_action_set_active (ta, FALSE);
		} else {
			/* word wrap default is on for plaintext and off for source code*/
			if (g_utf8_collate (document_get_mimetype (document_view_get_document (self->document_view)), "text/plain") == 0) {
				gtk_toggle_action_set_active (ta, TRUE);
			} else {
				gtk_toggle_action_set_active (ta, FALSE);
			}
		}
	}
	gtk_action_set_sensitive (gtk_ui_manager_get_action (self->ui_manager, "/menubar/edit/undo"), gtk_source_buffer_can_undo (GTK_SOURCE_BUFFER (document_view_get_document (self->document_view))));
	gtk_action_set_sensitive (gtk_ui_manager_get_action (self->ui_manager, "/menubar/edit/redo"), gtk_source_buffer_can_redo (GTK_SOURCE_BUFFER (document_view_get_document (self->document_view))));
}


static gboolean __lambda0 (GtkWidget* o, GdkEventFocus* e, BaseWindow* self)
{
	g_return_val_if_fail (o == NULL || GTK_IS_WIDGET (o), FALSE);
	base_window_save_scroll_position (self);
}


void base_window_connect_base_events (BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	g_signal_connect_object (self, "delete-event", (GCallback) base_window_on_close, self, 0);
	g_signal_connect_object (self->priv->searchentry, "key-press-event", (GCallback) base_window_on_searchentry_key_press, self, 0);
	g_signal_connect_object (self->priv->searchentry, "focus-out-event", (GCallback) base_window_on_searchentry_focus_out, self, 0);
	g_signal_connect_object (self->priv->searchentry, "search-activated", (GCallback) base_window_on_searchentry_search_activated, self, 0);
	g_signal_connect_object (self->priv->searchentry, "search-closed", (GCallback) base_window_on_searchentry_closed, self, 0);
	g_signal_connect_object (document_view_get_document (self->document_view), "mark-set", (GCallback) base_window_on_document_mark_set, self, 0);
	g_signal_connect_object (document_view_get_document (self->document_view), "can-undo", (GCallback) base_window_on_undo_redo_changed, self, 0);
	g_signal_connect_object (document_view_get_document (self->document_view), "can-redo", (GCallback) base_window_on_undo_redo_changed, self, 0);
	g_signal_connect_object (self->document_view, "focus-out-event", (GCallback) __lambda0, self, 0);
}


/* make this virtual once chaining is implemented */
void base_window_populate_base_window (BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	/* placeholder for the actual document view */
	GtkScrolledWindow* __temp12 = NULL;
	self->scrolled_window = (__temp12 = g_object_ref_sink (gtk_scrolled_window_new (NULL, NULL)), (self->scrolled_window == NULL ? NULL : (self->scrolled_window = (g_object_unref (self->scrolled_window), NULL))), __temp12);
	gtk_scrolled_window_set_policy (self->scrolled_window, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (self->scrolled_window, GTK_SHADOW_IN);
	GtkVBox* docbox = g_object_ref_sink (gtk_vbox_new (FALSE, 0));
	gtk_container_add (GTK_CONTAINER (docbox), GTK_WIDGET (self->scrolled_window));
	/* searchbar */
	GtkLabel* lbl = g_object_ref_sink (gtk_label_new ("Find:"));
	SearchEntry* __temp13 = NULL;
	self->priv->searchentry = (__temp13 = g_object_ref_sink (search_entry_new (lbl)), (self->priv->searchentry == NULL ? NULL : (self->priv->searchentry = (g_object_unref (self->priv->searchentry), NULL))), __temp13);
	GtkHBox* __temp14 = NULL;
	self->priv->searchbar = (__temp14 = g_object_ref_sink (gtk_hbox_new (FALSE, 0)), (self->priv->searchbar == NULL ? NULL : (self->priv->searchbar = (g_object_unref (self->priv->searchbar), NULL))), __temp14);
	gtk_widget_set_no_show_all (GTK_WIDGET (self->priv->searchbar), TRUE);
	gtk_box_set_spacing (GTK_BOX (self->priv->searchbar), 6);
	gtk_container_set_border_width (GTK_CONTAINER (self->priv->searchbar), 3);
	gtk_box_pack_start (GTK_BOX (self->priv->searchbar), GTK_WIDGET (lbl), FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (self->priv->searchbar), GTK_WIDGET (self->priv->searchentry), TRUE, TRUE, 0);
	/* document area */
	GtkVBox* document_area = g_object_ref_sink (gtk_vbox_new (FALSE, 0));
	gtk_box_pack_start (GTK_BOX (document_area), GTK_WIDGET (docbox), TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (document_area), GTK_WIDGET (self->priv->searchbar), FALSE, FALSE, 0);
	/* the side pane is used for project windows */
	GtkHPaned* __temp15 = NULL;
	self->priv->pane = (__temp15 = g_object_ref_sink (gtk_hpaned_new ()), (self->priv->pane == NULL ? NULL : (self->priv->pane = (g_object_unref (self->priv->pane), NULL))), __temp15);
	gtk_paned_set_position (GTK_PANED (self->priv->pane), 200);
	gtk_paned_add2 (GTK_PANED (self->priv->pane), GTK_WIDGET (document_area));
	/* statusbar */
	GtkHBox* statusbox = g_object_ref_sink (gtk_hbox_new (FALSE, 0));
	GtkStatusbar* __temp16 = NULL;
	self->priv->statusbar = (__temp16 = g_object_ref_sink (gtk_statusbar_new ()), (self->priv->statusbar == NULL ? NULL : (self->priv->statusbar = (g_object_unref (self->priv->statusbar), NULL))), __temp16);
	GtkStatusbar* __temp17 = NULL;
	self->priv->secondary_statusbar = (__temp17 = g_object_ref_sink (gtk_statusbar_new ()), (self->priv->secondary_statusbar == NULL ? NULL : (self->priv->secondary_statusbar = (g_object_unref (self->priv->secondary_statusbar), NULL))), __temp17);
	gtk_statusbar_set_has_resize_grip (self->priv->statusbar, FALSE);
	gtk_widget_set_size_request (GTK_WIDGET (self->priv->secondary_statusbar), 168, 0);
	gtk_box_pack_start (GTK_BOX (statusbox), GTK_WIDGET (self->priv->statusbar), TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (statusbox), GTK_WIDGET (self->priv->secondary_statusbar), FALSE, TRUE, 0);
	/* main box */
	GtkVBox* mainbox = g_object_ref_sink (gtk_vbox_new (FALSE, 0));
	gtk_box_pack_start (GTK_BOX (mainbox), gtk_ui_manager_get_widget (self->ui_manager, "/menubar"), FALSE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (mainbox), GTK_WIDGET (self->priv->pane), TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (mainbox), GTK_WIDGET (statusbox), FALSE, TRUE, 0);
	gtk_container_add (GTK_CONTAINER (self), GTK_WIDGET (mainbox));
	(docbox == NULL ? NULL : (docbox = (g_object_unref (docbox), NULL)));
	(lbl == NULL ? NULL : (lbl = (g_object_unref (lbl), NULL)));
	(document_area == NULL ? NULL : (document_area = (g_object_unref (document_area), NULL)));
	(statusbox == NULL ? NULL : (statusbox = (g_object_unref (statusbox), NULL)));
	(mainbox == NULL ? NULL : (mainbox = (g_object_unref (mainbox), NULL)));
}


static void base_window_flash_statusbar (BaseWindow* self, char* str)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	base_window_flash_statusbar_clear (self);
	gtk_statusbar_push (self->priv->statusbar, 0, str);
	self->priv->flash_timeout = g_timeout_add (3000, base_window_flash_statusbar_timeout, (gpointer) self);
}


static void base_window_flash_statusbar_clear (BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	gtk_statusbar_pop (self->priv->statusbar, 0);
	if (self->priv->flash_timeout > 0) {
		g_source_remove (self->priv->flash_timeout);
		self->priv->flash_timeout = 0;
	}
}


static gboolean base_window_flash_statusbar_timeout (BaseWindow* self)
{
	g_return_val_if_fail (IS_BASE_WINDOW (self), FALSE);
	gtk_statusbar_pop (self->priv->statusbar, 0);
	return FALSE;
}


static void base_window_show_about_dialog (BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	GdkPixbuf* logo = gdk_pixbuf_new_from_file (g_strconcat (config_get_datadir (), "pixmaps/scratchpad.svg", NULL), NULL);
	gtk_show_about_dialog (GTK_WINDOW (self), "name", "Scratchpad", "logo", logo, "authors", BASE_WINDOW_authors, "version", BASE_WINDOW_version, "license", "Scratchpad is free software; you can redistribute it and/or modify \nit under the terms of the GNU General Public License as published by \nthe Free Software Foundation; either version 2 of the License, or \n(at your option) any later version.\n\nScratchpad is distributed in the hope that it will be useful, \nbut WITHOUT ANY WARRANTY; without even the implied warranty of \nMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \nGNU General Public License for more details.\n\nYou should have received a copy of the GNU General Public License \nalong with Scratchpad; if not, write to the Free Software Foundation, Inc., \n51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA", "comments", "Text editor with attitude", "copyright", "Copyright © 2006 Daniel Borgmann", "website", "http://dborg.wordpress.com/scratchpad/", "website-label", "Visit the Scratchpad Homepage", NULL);
	(logo == NULL ? NULL : (logo = (g_object_unref (logo), NULL)));
}


static gboolean base_window_close (BaseWindow* self)
{
	g_return_val_if_fail (IS_BASE_WINDOW (self), FALSE);
	document_save (document_view_get_document (self->document_view));
	base_window_save_scroll_position (self);
	return FALSE;
}


static void base_window_on_undo_redo_changed (GObject* o, gboolean test, BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	g_return_if_fail (o == NULL || G_IS_OBJECT (o));
	base_window_update_menu_state (self);
}


static void base_window_on_document_mark_set (GObject* o, GtkTextIter* location, GtkTextMark* mark, BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	g_return_if_fail (o == NULL || G_IS_OBJECT (o));
	g_return_if_fail (mark == NULL || GTK_IS_TEXT_MARK (mark));
	gtk_statusbar_pop (self->priv->secondary_statusbar, 0);
	GtkTextIter iter;
	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document_view_get_document (self->document_view)), &iter, gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (document_view_get_document (self->document_view))));
	int row = gtk_text_iter_get_line (&iter) + 1;
	int col = gtk_text_iter_get_line_offset (&iter) + 1;
	char* __temp24 = NULL;
	char* __temp23 = NULL;
	char* __temp22 = NULL;
	char* __temp21 = NULL;
	char* __temp20 = NULL;
	gtk_statusbar_push (self->priv->secondary_statusbar, 0, (__temp24 = g_strconcat ((__temp22 = g_strconcat ((__temp21 = g_strconcat ("  Ln: ", (__temp20 = g_strdup_printf ("%i", row)), NULL)), ", Col: ", NULL)), (__temp23 = g_strdup_printf ("%i", col)), NULL)));
	(__temp24 == NULL ? NULL : (__temp24 = (g_free (__temp24), NULL)));
	(__temp23 == NULL ? NULL : (__temp23 = (g_free (__temp23), NULL)));
	(__temp22 == NULL ? NULL : (__temp22 = (g_free (__temp22), NULL)));
	(__temp21 == NULL ? NULL : (__temp21 = (g_free (__temp21), NULL)));
	(__temp20 == NULL ? NULL : (__temp20 = (g_free (__temp20), NULL)));
}


static void base_window_on_menu_item_select (GtkMenuItem* proxy, BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	g_return_if_fail (proxy == NULL || GTK_IS_MENU_ITEM (proxy));
	GtkAction* action = gtk_widget_get_action (GTK_WIDGET (proxy));
	g_return_if_fail (action != NULL);
	char* __temp25 = NULL;
	if ((g_object_get (G_OBJECT (action), "tooltip", &__temp25, NULL), __temp25) != NULL) {
		char* __temp26 = NULL;
		gtk_statusbar_push (self->priv->statusbar, gtk_statusbar_get_context_id (self->priv->statusbar, "tooltip"), (g_object_get (G_OBJECT (action), "tooltip", &__temp26, NULL), __temp26));
	}
}


static void base_window_on_menu_item_deselect (GtkMenuItem* proxy, BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	g_return_if_fail (proxy == NULL || GTK_IS_MENU_ITEM (proxy));
	gtk_statusbar_pop (self->priv->statusbar, gtk_statusbar_get_context_id (self->priv->statusbar, "tooltip"));
}


static void base_window_on_connect_proxy (GtkUIManager* manager, GtkAction* action, GtkWidget* proxy, BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	g_return_if_fail (manager == NULL || GTK_IS_UI_MANAGER (manager));
	g_return_if_fail (action == NULL || GTK_IS_ACTION (action));
	g_return_if_fail (proxy == NULL || GTK_IS_WIDGET (proxy));
	g_signal_connect_object ((GTK_ITEM (proxy)), "select", (GCallback) base_window_on_menu_item_select, self, 0);
	g_signal_connect_object ((GTK_ITEM (proxy)), "deselect", (GCallback) base_window_on_menu_item_deselect, self, 0);
}


static void base_window_on_disconnect_proxy (GtkUIManager* manager, GtkAction* action, GtkWidget* proxy, BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	g_return_if_fail (manager == NULL || GTK_IS_UI_MANAGER (manager));
	g_return_if_fail (action == NULL || GTK_IS_ACTION (action));
	g_return_if_fail (proxy == NULL || GTK_IS_WIDGET (proxy));
	g_signal_connect_object ((GTK_ITEM (proxy)), "select", (GCallback) base_window_on_menu_item_select, self, 0);
	g_signal_connect_object ((GTK_ITEM (proxy)), "deselect", (GCallback) base_window_on_menu_item_deselect, self, 0);
}


static void base_window_on_searchentry_search_activated (GObject* o, SearchEventArgs* args, BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	g_return_if_fail (o == NULL || G_IS_OBJECT (o));
	g_return_if_fail (args == NULL || IS_SEARCH_EVENT_ARGS (args));
	if (search_event_args_get_mode (args) == SEARCH_MODE_GO_TO_LINE) {
		document_view_go_to_line (self->document_view, atoi (args->search_text));
		return;
	}
	if (search_event_args_get_mode (args) == SEARCH_MODE_REPLACE) {
		document_view_replace (self->document_view, args->search_text);
		return;
	}
	GtkTextIter i;
	GtkTextIter j;
	GtkTextIter start;
	gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (document_view_get_document (self->document_view)), &i, &j);
	start = ((args->next_result && !args->backwards) ? j : i);
	/* search case insensitive if string is all lower case*/
	char* __temp27 = NULL;
	gboolean __temp28;
	gboolean match_case = (__temp28 = ((g_utf8_collate (args->search_text, (__temp27 = g_utf8_casefold (args->search_text, -1))) != 0) ? TRUE : FALSE), (__temp27 == NULL ? NULL : (__temp27 = (g_free (__temp27), NULL))), __temp28);
	if (!document_view_find (self->document_view, args->search_text, start, match_case, args->backwards, TRUE)) {
		char* __temp30 = NULL;
		char* __temp29 = NULL;
		base_window_flash_statusbar (self, (__temp30 = g_strconcat ((__temp29 = g_strconcat ("No search result for \"", args->search_text, NULL)), "\"", NULL)));
		(__temp30 == NULL ? NULL : (__temp30 = (g_free (__temp30), NULL)));
		(__temp29 == NULL ? NULL : (__temp29 = (g_free (__temp29), NULL)));
	} else {
		base_window_flash_statusbar_clear (self);
	}
}


static gboolean base_window_on_searchentry_key_press (GObject* o, GdkEventKey* key, BaseWindow* self)
{
	g_return_val_if_fail (IS_BASE_WINDOW (self), FALSE);
	g_return_val_if_fail (o == NULL || G_IS_OBJECT (o), FALSE);
	/* unfocus and hide the searchbar when escape is pressed*/
	if (g_utf8_collate (gdk_keyval_name (key->keyval), "Escape") == 0) {
		gtk_widget_grab_focus (GTK_WIDGET (self->document_view));
	}
	return FALSE;
}


static gboolean base_window_on_searchentry_focus_out (GObject* o, GdkEventAny* e, BaseWindow* self)
{
	g_return_val_if_fail (IS_BASE_WINDOW (self), FALSE);
	g_return_val_if_fail (o == NULL || G_IS_OBJECT (o), FALSE);
	gtk_widget_hide (GTK_WIDGET (self->priv->searchbar));
	return FALSE;
}


static void base_window_on_searchentry_closed (GObject* o, BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	g_return_if_fail (o == NULL || G_IS_OBJECT (o));
	gtk_widget_hide (GTK_WIDGET (self->priv->searchbar));
}


static void base_window_on_action_activated (GObject* o, GtkAction* action, BaseWindow* self)
{
	g_return_if_fail (IS_BASE_WINDOW (self));
	g_return_if_fail (o == NULL || G_IS_OBJECT (o));
	g_return_if_fail (action == NULL || GTK_IS_ACTION (action));
	GtkToggleAction* ta = NULL;
	/* TODO: replace with switch statement once it works with strings
	 File menu */
	char* __temp33 = NULL;
	if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp33, NULL), __temp33), "open-folder") == 0) {
		char* __temp34 = NULL;
		g_spawn_command_line_async ((__temp34 = g_strconcat ("gnome-open ", document_get_folder (document_view_get_document (self->document_view)), NULL)), NULL);
		(__temp34 == NULL ? NULL : (__temp34 = (g_free (__temp34), NULL)));
	} else {
		char* __temp35 = NULL;
		if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp35, NULL), __temp35), "close") == 0) {
			g_signal_emit_by_name (G_OBJECT (self), "destroy");
		} else {
			/* Edit menu */
			char* __temp36 = NULL;
			if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp36, NULL), __temp36), "undo") == 0) {
				if (gtk_source_buffer_can_undo (GTK_SOURCE_BUFFER (document_view_get_document (self->document_view)))) {
					gtk_source_buffer_undo (GTK_SOURCE_BUFFER (document_view_get_document (self->document_view)));
				}
				base_window_update_menu_state (self);
			} else {
				char* __temp37 = NULL;
				if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp37, NULL), __temp37), "redo") == 0) {
					if (gtk_source_buffer_can_redo (GTK_SOURCE_BUFFER (document_view_get_document (self->document_view)))) {
						gtk_source_buffer_redo (GTK_SOURCE_BUFFER (document_view_get_document (self->document_view)));
					}
					base_window_update_menu_state (self);
				} else {
					char* __temp38 = NULL;
					if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp38, NULL), __temp38), "cut") == 0) {
						document_view_block_cut_clipboard (self->document_view, FALSE);
					} else {
						char* __temp39 = NULL;
						if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp39, NULL), __temp39), "copy") == 0) {
							document_view_block_copy_clipboard (self->document_view, FALSE);
						} else {
							char* __temp40 = NULL;
							if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp40, NULL), __temp40), "paste") == 0) {
								document_view_block_paste_clipboard (self->document_view);
							} else {
								char* __temp41 = NULL;
								if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp41, NULL), __temp41), "block-cut") == 0) {
									document_view_block_cut_clipboard (self->document_view, TRUE);
								} else {
									char* __temp42 = NULL;
									if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp42, NULL), __temp42), "block-copy") == 0) {
										document_view_block_copy_clipboard (self->document_view, TRUE);
									} else {
										char* __temp43 = NULL;
										if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp43, NULL), __temp43), "find") == 0) {
											search_entry_set_mode (self->priv->searchentry, SEARCH_MODE_TEXT);
											gtk_widget_set_no_show_all (GTK_WIDGET (self->priv->searchbar), FALSE);
											gtk_widget_show_all (GTK_WIDGET (self->priv->searchbar));
											gtk_widget_grab_focus (GTK_WIDGET (self->priv->searchentry));
										} else {
											char* __temp44 = NULL;
											if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp44, NULL), __temp44), "replace") == 0) {
												search_entry_set_mode (self->priv->searchentry, SEARCH_MODE_REPLACE);
												gtk_widget_set_no_show_all (GTK_WIDGET (self->priv->searchbar), FALSE);
												gtk_widget_show_all (GTK_WIDGET (self->priv->searchbar));
												gtk_widget_grab_focus (GTK_WIDGET (self->priv->searchentry));
											} else {
												char* __temp45 = NULL;
												if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp45, NULL), __temp45), "goto-line") == 0) {
													search_entry_set_maxlines (self->priv->searchentry, gtk_text_buffer_get_line_count (GTK_TEXT_BUFFER (document_view_get_document (self->document_view))));
													search_entry_set_mode (self->priv->searchentry, SEARCH_MODE_SILENT);
													char* __temp46 = NULL;
													gtk_entry_set_text (GTK_ENTRY (self->priv->searchentry), (__temp46 = g_strdup_printf ("%i", document_view_get_current_line (self->document_view))));
													(__temp46 == NULL ? NULL : (__temp46 = (g_free (__temp46), NULL)));
													search_entry_set_mode (self->priv->searchentry, SEARCH_MODE_GO_TO_LINE);
													gtk_widget_set_no_show_all (GTK_WIDGET (self->priv->searchbar), FALSE);
													gtk_widget_show_all (GTK_WIDGET (self->priv->searchbar));
													gtk_widget_grab_focus (GTK_WIDGET (self->priv->searchentry));
												} else {
													char* __temp47 = NULL;
													if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp47, NULL), __temp47), "select-all") == 0) {
														document_view_select_all (self->document_view);
													} else {
														char* __temp48 = NULL;
														if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp48, NULL), __temp48), "new-line") == 0) {
															document_view_insert_new_line (self->document_view);
														} else {
															/* Tags submenu */
															char* __temp49 = NULL;
															if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp49, NULL), __temp49), "tag-add") == 0) {
																document_view_tag_selection (self->document_view);
															} else {
																char* __temp50 = NULL;
																if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp50, NULL), __temp50), "tag-remove") == 0) {
																	document_view_remove_tags (self->document_view);
																} else {
																	char* __temp51 = NULL;
																	if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp51, NULL), __temp51), "tag-search") == 0) {
																		document_view_tag_search_results (self->document_view, search_entry_get_last_search (self->priv->searchentry));
																	} else {
																		char* __temp52 = NULL;
																		if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp52, NULL), __temp52), "tag-add-and-search") == 0) {
																			document_view_tag_and_find_next (self->document_view);
																		} else {
																			/* View menu */
																			char* __temp53 = NULL;
																			if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp53, NULL), __temp53), "line-numbers") == 0) {
																				gtk_source_view_set_show_line_numbers (GTK_SOURCE_VIEW (self->document_view), !gtk_source_view_get_show_line_numbers (GTK_SOURCE_VIEW (self->document_view)));
																				settings_set_show_line_numbers (gtk_source_view_get_show_line_numbers (GTK_SOURCE_VIEW (self->document_view)));
																			} else {
																				char* __temp54 = NULL;
																				if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp54, NULL), __temp54), "margin") == 0) {
																					gtk_source_view_set_show_margin (GTK_SOURCE_VIEW (self->document_view), !gtk_source_view_get_show_margin (GTK_SOURCE_VIEW (self->document_view)));
																					settings_set_show_margin (gtk_source_view_get_show_margin (GTK_SOURCE_VIEW (self->document_view)));
																				} else {
																					/* Document menu */
																					char* __temp55 = NULL;
																					if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp55, NULL), __temp55), "word-wrap") == 0) {
																						gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (self->document_view), ((gtk_toggle_action_get_active ((GTK_TOGGLE_ACTION (action)))) ? GTK_WRAP_WORD : GTK_WRAP_NONE));
																					} else {
																						/* Help menu */
																						char* __temp56 = NULL;
																						if (g_utf8_collate ((g_object_get (G_OBJECT (action), "name", &__temp56, NULL), __temp56), "about") == 0) {
																							base_window_show_about_dialog (self);
																						}
																					}
																				}
																			}
																		}
																	}
																}
															}
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
}


static gboolean base_window_on_close (GObject* o, GdkEventConfigure* e, BaseWindow* self)
{
	g_return_val_if_fail (IS_BASE_WINDOW (self), FALSE);
	g_return_val_if_fail (o == NULL || G_IS_OBJECT (o), FALSE);
	return base_window_close (self);
}


static GObject * base_window_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties)
{
	GObject * obj;
	BaseWindowClass * klass;
	GObjectClass * parent_class;
	klass = BASE_WINDOW_CLASS (g_type_class_peek (TYPE_BASE_WINDOW));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	BaseWindow * self = BASE_WINDOW (obj);
	{
		gtk_window_set_position (GTK_WINDOW (self), GTK_WIN_POS_CENTER);
		GtkClipboard* __temp59 = NULL;
		GtkClipboard* __temp58 = NULL;
		base_window_clipboard = (__temp59 = (__temp58 = gtk_clipboard_get (gdk_atom_intern ("CLIPBOARD", FALSE)), (__temp58 == NULL ? __temp58 : g_object_ref (__temp58))), (base_window_clipboard == NULL ? NULL : (base_window_clipboard = (g_object_unref (base_window_clipboard), NULL))), __temp59);
		base_window_setup_ui_manager (self);
	}
	return obj;
}


static void base_window_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	BaseWindow * self = BASE_WINDOW (object);
	switch (property_id) {
	}
}


static void base_window_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	BaseWindow * self = BASE_WINDOW (object);
	switch (property_id) {
	}
}


static void base_window_class_init (BaseWindowClass * klass)
{
	g_type_class_add_private (klass, sizeof (BaseWindowPrivate));
	G_OBJECT_CLASS (klass)->get_property = base_window_get_property;
	G_OBJECT_CLASS (klass)->set_property = base_window_set_property;
	G_OBJECT_CLASS (klass)->constructor = base_window_constructor;
	G_OBJECT_CLASS (klass)->dispose = base_window_dispose;
	BASE_WINDOW_CLASS (klass)->is_document = base_window_real_is_document;
}


static void base_window_init (BaseWindow * self)
{
	self->priv = BASE_WINDOW_GET_PRIVATE (self);
}


static void base_window_dispose (GObject * obj)
{
	BaseWindow * self = BASE_WINDOW (obj);
	(self->scrolled_window == NULL ? NULL : (self->scrolled_window = (g_object_unref (self->scrolled_window), NULL)));
	(self->document_view == NULL ? NULL : (self->document_view = (g_object_unref (self->document_view), NULL)));
	(self->ui_manager == NULL ? NULL : (self->ui_manager = (g_object_unref (self->ui_manager), NULL)));
	(self->priv->about_dialog == NULL ? NULL : (self->priv->about_dialog = (g_object_unref (self->priv->about_dialog), NULL)));
	(self->priv->pane == NULL ? NULL : (self->priv->pane = (g_object_unref (self->priv->pane), NULL)));
	(self->priv->searchbar == NULL ? NULL : (self->priv->searchbar = (g_object_unref (self->priv->searchbar), NULL)));
	(self->priv->searchentry == NULL ? NULL : (self->priv->searchentry = (g_object_unref (self->priv->searchentry), NULL)));
	(self->priv->statusbar == NULL ? NULL : (self->priv->statusbar = (g_object_unref (self->priv->statusbar), NULL)));
	(self->priv->secondary_statusbar == NULL ? NULL : (self->priv->secondary_statusbar = (g_object_unref (self->priv->secondary_statusbar), NULL)));
	BaseWindowClass * klass;
	GObjectClass * parent_class;
	klass = BASE_WINDOW_CLASS (g_type_class_peek (TYPE_BASE_WINDOW));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType base_window_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (BaseWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) base_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BaseWindow), 0, (GInstanceInitFunc) base_window_init };
		g_define_type_id = g_type_register_static (GTK_TYPE_WINDOW, "BaseWindow", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	}
	return g_define_type_id;
}



